function lecteur_norm, filenames,infonames, lecteurs, varid, output_varid,transfos, ether_norm_struct, daterange=daterange, lonrange=lonrange, latrange=latrange, param_aux=param_aux,densite=densite,erreur=erreur
;
; filenames : tableau de string
;
; infonames : tableau de string
; lecteurs : tableau de string
; varid : tableau de structures du type [{variable:'Aerosol', nature:'Mes', subvariable:'1018IR', unite:'m/s'},{variable:'Altitude',, nature:'Mes', subvariable:''}]
;output_varid : tableau de structure. parametres en sortie du lecteur (apres transformation) 
; transfos : tableau de structures [ptr_new([{nom:'t9',aux1:4,aux2:999},{nom:'c1',aux1:999,aux2:999}],ptr_new([{nom:'c1',aux1:999,aux2:999},{nom:'t4',aux1:4,aux2:999},{nom:'c2',aux1:999,aux2:999}]]
; ether_norm_struct :
; Tags de la structure ETHER_NORM_STRUCT
; ---------------------------------
;   nb_prof        (scalaire entier):
;       nombre de profils
;       nombre de profils total issu de chaque fichier de donnees trait et inclus dans les plages daterange, lonrange, latrange
;   nb_var         (scalaire entier):
;       nombre de variables
;       nombre de variables en sortie = N_elements(output_varid)
;   nb_niv         (scalaire entier):
;       nombre de niveaux verticaux des profils
;       nombre de niveaux MAX issu de chaque fichier de donnees trait
;       Nota : si le nombre de niveaux est differents selon le fichier de donnee traitee , mettre des valeurs 999.999 dans tableau aux niveaux non definis
;   date           (tableau de chaines de caractres de dimension nbprof):
;       date de chacun des profils au format AAAAMMJJhhmmss
;   latitude       (tableau de reels de dimension nbprof):
;       latitude de chacun des profils (en , entre -90 au S et +90 au N)
;   longitude      (tableau de reels de dimension nbprof):
;       longitude de chacun des profils (en , entre -180  l'W et +180  l'E)
;   numorb         tableau de chaines de caracteres de dimension nbprof):
;       numero d'orbite pour chacun des profils
;       dans le cas d'instruments embarques sur satellite
;   zensol         (tableau de reels de dimension nbprof):
;       angle zenithal solaire pour chacun des profils
;   comment        (tableau de chaines de caracteres de dimension nbprof):
;       informations specifiques aux fichiers
;   variable       (tableau de chaines de caracteres de dimension nb_var):
;       liste des variables lues dans le fichier
;       (especes chimiques ou parametres physiques)
;   subvariable    (tableau de chaines de caracteres de dimension nb_var):
;       liste des sous-variables (information complementaire  propos des variables)
;   unite          (tableau de chaines de caracteres de dimension nb_var):
;       unite dans laquelle est exprimee chacune des variables
;   nature         (tableau de chaines de caracteres de dimension nb_var):
;       nature de chacune des variables (provenant de mesures ou d'un modele)
;   tableau         (tableau de reels de dimension nb_prof x nb_var x nb_niv):
;       valeurs prises par chacune des variables, par niveau vertical et par profil
;
; NOTA : pour chaque variable de sortie (i) decrite dans output_varid , il existe un index (idx) unique telque :
; ether_norm_struct.variable[idx]=output_varid[i].variable
; ether_norm_struct.subvariable[idx]=output_varid[i].subvariable
; ether_norm_struct.nature[idx]=output_varid[i].nature
; ether_norm_struct.unite[idx]]=output_varid[i].unite
; ether_norm_struct.tableau[*,idx,*] : ensemble des valeurs calculee pour la variable (i) par profil et par niveau
; 
; daterange=[20040601120054,20040602120054]
; lonrange=[20,180]
; param_aux=param_aux ; ex : [ptr_new({nomfile:<file>,nomvariable:'Press',nomsubvariable:'.',nomnature:'Mes', nominfo:'<file>'},       
;                                    {nomfile:aux_input,nomvariable:'Temp',nomsubvariable:'.',nomnature:'Mes', nominfo:'<file>'}]))
; densite=densite ; 1 ou 0
; erreur=erreur
;

if densite eq 3 then begin

ether_struct = CALL_FUNCTION(lecteurs[0],filenames[0])

endif else begin


;setdata,systime(),0,1 
IF N_ELEMENTS(filenames) eq 0 THEN BEGIN
    erreur='At least one file must be specified'
    print,erreur
    RETURN,'-1'
ENDIF
IF N_ELEMENTS(infonames) eq 0 THEN BEGIN
    erreur='At least one information file must be specified'
    print,erreur
    RETURN,'-1'
ENDIF
IF N_ELEMENTS(lecteurs) eq 0 THEN BEGIN
    erreur='At least one reader program must be specified'
    print,erreur
    RETURN,'-1'
ENDIF
IF N_ELEMENTS(infonames) ne N_ELEMENTS(filenames) THEN BEGIN
    erreur='The number of information files must be equal to the number of files'
    print,erreur
    RETURN,'-1'
ENDIF
IF N_ELEMENTS(varid) ne N_ELEMENTS(transfos) THEN BEGIN
    erreur='The number of transformations must be equal to the number of variables'
    print,erreur
    RETURN,'-1'
ENDIF
IF N_ELEMENTS(varid) lt 2 THEN BEGIN
    erreur='At least 2 variables must be specified'
    print,erreur
    RETURN,'-1'
ENDIF
IF N_ELEMENTS(output_varid) lt 2 THEN BEGIN
    erreur='At least 2 variables must be specified for the output variables'
    print,erreur
    RETURN,'-1'
ENDIF
IF N_ELEMENTS(param_aux) ne N_ELEMENTS(filenames) THEN BEGIN
    IF N_ELEMENTS(param_aux) eq 1 THEN param_aux = REPLICATE(param_aux, N_ELEMENTS(filenames)) ELSE BEGIN
	if n_elements(param_aux) eq 0 then param_aux = replicate([ptr_new([{nomfile:'',nomvariable:'',nomsubvariable:'',nomnature:'',nominfo:''}])],n_elements(filenames)) else begin
	erreur='The number of auxiliary parameters must be equal to the number of files'
   	print,erreur
	RETURN,'-1'
	endelse
endelse
ENDIF

IF SIZE(output_varid[0],/type) ne 8 THEN BEGIN
    erreur='The output variables must be contained in a structure'
    print,erreur
    RETURN,'-1'
END
tag = TAG_NAMES(output_varid[0])
IF TOTAL(STRMATCH(tag,'variable',/fold_case)) eq 0 THEN BEGIN 
    erreur="The tag 'variable' is not defined in output variables"
    print,erreur
    RETURN,'-1'
ENDIF
IF TOTAL(STRMATCH(tag,'subvariable',/fold_case)) eq 0 THEN BEGIN 
    erreur="The tag 'subvariable' is not defined in output variables"
    print,erreur
    RETURN,'-1'
ENDIF
IF TOTAL(STRMATCH(tag,'nature',/fold_case)) eq 0 THEN BEGIN
    erreur="The tag 'nature' is not defined in output variables"
    print,erreur
    RETURN,'-1'
ENDIF

IF SIZE(varid[0],/type) ne 8 THEN BEGIN
    erreur='The variables must be contained in a structure'
    print,erreur
    RETURN,'-1'
END
tag = TAG_NAMES(varid[0])
IF TOTAL(STRMATCH(tag,'variable',/fold_case)) eq 0 THEN BEGIN
    erreur="The tag 'variable' is not defined"
    print,erreur
    RETURN,'-1'
ENDIF
IF TOTAL(STRMATCH(tag,'subvariable',/fold_case)) eq 0 THEN BEGIN
    erreur="The tag 'subvariable' is not defined"
    print,erreur
    RETURN,'-1'
ENDIF
IF TOTAL(STRMATCH(tag,'nature',/fold_case)) eq 0 THEN BEGIN
    erreur="The tag 'nature' is not defined"
    print,erreur
    RETURN,'-1'
ENDIF


IF N_ELEMENTS(daterange) lt 2 THEN daterange=[ulong64(0),ulong64(30000000000000)]
IF N_ELEMENTS(lonrange) lt 2 THEN lonrange=[-180,360]
IF N_ELEMENTS(latrange) lt 2 THEN latrange=[-90,90]


; initialisation des variables de sortie
varx = ptr_new()
vary = ptr_new()

vardate = ptr_new()
varlatitude = ptr_new()
varlongitude = ptr_new()
varnumorb = ptr_new()
varzensol = ptr_new()
varcomment = ptr_new()


; boucle sur les fichiers
erreur=''
erreur0=''
niveau_norm=1
tot_nbprof=0
FOR ifile = 0, N_ELEMENTS(filenames)-1 DO BEGIN
	; filenames[ifile] est bien accessible ?
	IF FILE_TEST(filenames[ifile],/read) eq 0 THEN print,"The file "+filenames[ifile]+" does not exist or is not readable" ELSE BEGIN
	
	; lecture du fichier
	;print,'filename=',filenames[ifile] 
        ether_struct = CALL_FUNCTION(lecteurs[0],filenames[ifile],infonames[ifile],densite=densite,paramaux=*param_aux[ifile])
	print,'size(ether_struct->reader',size(ether_struct.tableau)
	;print,'******************'
;print,ether_struct.variable
;print,'*******************'

;print,'******************'
;print,ether_struct.subvariable
;nprint,'*******************'

;print,'******************'
;print,ether_struct.nature
;print,'*******************'

	
	; extraction des variables; combien il y en a, d'abord ?
	
        nvar = N_ELEMENTS(varid)
	; declaration array de pointeurs car la dimension des profils n'est pas connue
        var = ptrarr(nvar)

	; commence par la fin -> sont stockees les variables auxiliaires et au passage application des tranformations 
	noprof=0
	
        FOR ivar = nvar-1,0,-1 DO BEGIN
;		print,'ivar=',ivar
;		print,'ether_struct.nature=',ether_struct.nature
;		print,'varid[ivar].nature=',varid[ivar].nature
;		print,'****************************************'
;		print,'ether_struct.variable=',ether_struct.variable
;		print,'varid[ivar].variable=',varid[ivar].variable


;		if ether_struct.nature eq varid[ivar].nature then begin
;			print,'OUI NATURE'
;		endif else begin
;			print, 'NON NATURE'
;		endelse

;		if ether_struct.subvariable eq varid[ivar].subvariable then begin
;                        print, 'OUI SUB'
;                endif else begin
;                        print, 'NON SUB'
;                endelse
		

        	idx = WHERE(STRMATCH(ether_struct.variable,varid[ivar].variable) ne 0 and $
                        ether_struct.nature eq varid[ivar].nature and $
                        ether_struct.subvariable eq varid[ivar].subvariable,count)
			;print,'variable=',ether_struct.variable
			;print,'longitude=',ether_struct.longitude
			;print,'latitude=',ether_struct.latitude
			
            	IF count eq 0 and varid[ivar].variable ne 'phi' and varid[ivar].nature ne 'ARL' THEN BEGIN
			erreur0=erreur0+"!cThe variable ("+varid[ivar].variable+","$
				+varid[ivar].nature+","+varid[ivar].subvariable+") is not available in the file  "+filenames[ifile]
			noprof=1
			print,'!!>>',erreur0
		ENDIF ELSE BEGIN
			isel = WHERE(ether_struct.longitude ge lonrange[0],count)
			if count eq 0 then begin
				print,'pb lonmin'
			endif

			isel = WHERE(ether_struct.longitude le lonrange[1],count)
                        if count eq 0 then begin
                                print,'pb lonmax'
                        endif

			isel = WHERE(ether_struct.latitude ge latrange[0],count)
                        if count eq 0 then begin
                                print,'pb latmin'
                        endif

			isel = WHERE(ether_struct.latitude le latrange[1],count)
                        if count eq 0 then begin
				print,'pb latmax'
                        endif

			isel = WHERE( ulong64(ether_struct.date) ge daterange[0],count)
                        if count eq 0 then begin
                                print,'pb datemin'
                        endif

			isel = WHERE( ulong64(ether_struct.date) le daterange[1],count)
                        if count eq 0 then begin
                                print,'pb datemax'
                        endif

                	isel = WHERE(ether_struct.longitude ge lonrange[0] and ether_struct.longitude le lonrange[1] and $
				ether_struct.latitude ge latrange[0] and ether_struct.latitude le latrange[1] and $
				ulong64(ether_struct.date) ge daterange[0] and ulong64(ether_struct.date) le daterange[1] ,count)

				 
			IF count eq 0 THEN BEGIN
				erreur='No drawing in the selected range of date/latitude/longitude!c -> Directory : '+$
					strmid(filenames[ifile],0,strpos(filenames[ifile],'/',/reverse_search))+'!c -> File : '+$
					strmid(filenames[ifile],strpos(filenames[ifile],'/',/reverse_search)+1)

    		    		print,erreur
				erreur0=erreur0+'!c'+erreur
				noprof=1
				break
		        ENDIF

			IF densite eq 0 THEN nbniv = (size(ether_struct.tableau))[3] else nbniv = 1
			
			; Lancement du script ARLETTY
	                IF varid[ivar].nature eq 'ARL' THEN BEGIN
				arletty,ether_struct,isel,count,varid,ivar,numreq,erreur=erreur
				erreur0=erreur0+'!c'+erreur
			ENDIF ELSE BEGIN
				IF varid[ivar].variable eq 'phi' THEN BEGIN
	       		                dumvar = fltarr(count,nbniv)
       		        	        FOR iniv = 0L, nbniv-1 DO BEGIN
						dumvar[*,iniv]=ether_struct.latitude[isel]
					ENDFOR	
		                ENDIF ELSE BEGIN 
	;				print,'size(n_elements)=',size(ether_struct.tableau)
	;				print,'idx=',idx
					IF n_elements(ether_struct.tableau[isel,idx,*]) eq 1 THEN BEGIN
						dumvar =ether_struct.tableau[isel,idx,*] 
					ENDIF ELSE BEGIN
	;					print,'count=',count
	;					print,'nbniv=',nbniv
	;					print,'size(ether_struct.tableau[isel,idx,*]=',size(ether_struct.tableau[isel,idx,*])
	;					print,'n_elements(ether_struct.tableau[isel,idx,*])=",n_elements(ether_struct.tableau[isel,idx,*])
						dumvar = REFORM(ether_struct.tableau[isel,idx,*],count,nbniv)
					ENDELSE
				ENDELSE
			ENDELSE

;			print,'ivar=',ivar,' | ','dumvar=',dumvar	

			nbprof=count
			
			if ivar eq 0 then begin
				tot_nbprof = tot_nbprof + nbprof
			endif

			;transformations/conversions
	                ntransfo = N_ELEMENTS(*transfos[ivar])

	                FOR itransfo = 0, ntransfo-1 DO BEGIN
				
				idx = (*transfos[ivar])[itransfo].aux1

				IF idx ge 2 and idx lt nvar THEN BEGIN

		                        IF STREGEX(varid[idx].variable,'phi',/fold_case) ge 0 THEN aux1=ether_struct.latitude ELSE aux1=*var[idx] 

				ENDIF ELSE aux1=dumvar

		                idx = (*transfos[ivar])[itransfo].aux2
				IF idx ge 2 and idx lt nvar THEN BEGIN
					IF STREGEX(varid[idx].variable,'phi',/fold_case) ge 0 THEN aux2=ether_struct.latitude ELSE aux2=*var[idx] 
			        ENDIF ELSE aux2=dumvar

				idx = WHERE(dumvar ne 999 and aux1 ne 999 and aux2 ne 999,count)
				
				IF count ne 0 THEN BEGIN
					FOR iprof = 0L, nbprof-1 DO BEGIN 
						dumvar[iprof,*] = CALL_FUNCTION((*transfos[ivar])[itransfo].nom,dumvar[iprof,*],aux1=aux1[iprof,*],$
					aux2=aux2[iprof,*])
						IF n_elements(dumvar[iprof,*]) gt niveau_norm THEN niveau_norm=n_elements(dumvar[iprof,*])
					ENDFOR
	                       		IF densite eq 0 THEN BEGIN
						var[ivar] = ptr_new(reform(dumvar,nbprof,(size(ether_struct.tableau))[3]))
									
					ENDIF ELSE BEGIN
						IF n_elements(dumvar) eq 1 THEN BEGIN
							var[ivar] = ptr_new(dumvar) 
						ENDIF ELSE var[ivar] = ptr_new(REFORM(dumvar,nbprof,1))
		                        ENDELSE
			
		                ENDIF ELSE BEGIN	
					print,'err0=',erreur0
			                erreur0=erreur0+'!cImpossible transformation : '+(*transfos[ivar])[itransfo].nom
			                var[ivar] = ptr_new(999)
			        ENDELSE
		        ENDFOR
			
		ENDELSE
	ENDFOR
	donnee=''
	IF noprof eq 0 THEN BEGIN
	        varx = [varx, var[0]]
        	vary = [vary, var[1]]
		
		varlongitude = [varlongitude, ptr_new(reform(ether_struct.longitude[isel]))]
		varlatitude = [varlatitude, ptr_new(reform(ether_struct.latitude[isel]))]
		vardate = [vardate, ptr_new(reform(ether_struct.date[isel]))]
		varnumorb = [varnumorb, ptr_new(reform(ether_struct.numorb[isel]))]
		varzensol = [varzensol, ptr_new(reform(ether_struct.zensol[isel]))]
		varcomment = [varcomment, ptr_new(reform(ether_struct.zensol[isel]))]
	ENDIF
    ENDELSE
ENDFOR

if n_elements(varx) le 1 then return,'-1'
;print,'varx=',varx
varx = varx[1:*]
vary = vary[1:*]
varlongitude = varlongitude[1:*]
varlatitude = varlatitude[1:*]
vardate = vardate[1:*]
varnumorb = varnumorb[1:*]
varzensol = varzensol[1:*]
varcomment = varcomment[1:*]

pl_varx=varx

;if n_elements(varx) le 1 then return,'-1

pl_vary=vary
pl_varlongitude=varlongitude
pl_varlatitude=varlatitude
pl_vardate=vardate
pl_varnumorb=varnumorb
pl_varzensol=varzensol
pl_varcomment=varcomment

;dimension 2 de ether_norm_struct=nombre de variable en sortie
nbvar=n_elements(output_varid)

;calcul du nombre total de profils
nivtab=tot_nbprof

; construction de la structure ether normalisee
ether_norm_struct = {nb_prof : nivtab, nb_var : nbvar, nb_niv : niveau_norm, $
                date : strarr(nivtab), $
                latitude : fltarr(nivtab), longitude : fltarr(nivtab), $
                numorb : strarr(nivtab), zensol : fltarr(nivtab), $
                comment : strarr(nivtab), $
                variable : strarr(n_elements(output_varid)), subvariable : strarr(n_elements(output_varid)), $
                unite : strarr(n_elements(output_varid)), nature: strarr(n_elements(output_varid)), $
		tableau : fltarr(nivtab,nbvar,niveau_norm)}

indice=0


;on parcourt le tableaux de points transformes des profils retenus
;pour remplir ether_norm_struct.tableau 

for n=0L,n_elements(pl_varx)-1 do begin
        nbprof=(size(*pl_varx[n],/dimensions))[0]
	
        for iprof=0L,nbprof-1 do begin
		m=0L
                for m=0L,n_elements((*pl_varx[n])[iprof,*])-1 do begin
                        ether_norm_struct.tableau[indice,0,m]=(*pl_varx[n])[iprof,m]
			ether_norm_struct.tableau[indice,1,m]=(*pl_vary[n])[iprof,m]
;			ether_norm_struct.tableau[indice,2,m]=(*pl_vary[n])[iprof,m]
                endfor
			
		ether_norm_struct.longitude[indice]=(*pl_varlongitude[n])[iprof]
		ether_norm_struct.latitude[indice]=(*pl_varlatitude[n])[iprof]
		ether_norm_struct.zensol[indice]=(*pl_varzensol[n])[iprof]
		ether_norm_struct.date[indice]=(*pl_vardate[n])[iprof]
		ether_norm_struct.numorb[indice]=(*pl_varnumorb[n])[iprof]
		ether_norm_struct.comment[indice]=(*pl_varcomment[n])[iprof]	
			
		;si ce profil n'a pas le nombre de niveau maximum, initialisation valeur a 999.999
		if m lt niveau_norm-1 then begin
			for k=m, niveau_norm-1 do begin
				ether_norm_struct.tableau[indice,0,k]=999.999
				ether_norm_struct.tableau[indice,1,k]=999.999
;				ether_norm_struct.tableau[indice,2,k]=999.999
			endfor 
		endif
		indice=indice+1
        endfor
		
endfor

;on renseigne ether_norm_struct des variables de sortie listees dans output_varid
;print,'n_elements(output_varid)=',output_varid[2]
for n=0L,n_elements(output_varid)-1 do begin
	ether_norm_struct.variable[n]=output_varid[n].variable
	ether_norm_struct.subvariable[n]=output_varid[n].subvariable
	ether_norm_struct.unite[n]=output_varid[n].unite
	ether_norm_struct.nature[n]=output_varid[n].nature
endfor
erreur=erreur0
;setdata,systime(),1,1
;print,'*******VARIABLES**********'
;print,ether_norm_struct.tableau
;print,'**************************'

endelse
RETURN, '0'
END

